---
title: llarp/layers/platform/deprecated_driver_pimpl.hpp

---

# llarp/layers/platform/deprecated_driver_pimpl.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::platform::NameLookupJob_Deprecated](Classes/classllarp_1_1layers_1_1platform_1_1NameLookupJob__Deprecated.md)**  |
| struct | **[llarp::layers::platform::DNSReplier](Classes/structllarp_1_1layers_1_1platform_1_1DNSReplier.md)**  |
| class | **[llarp::layers::platform::DNS_NameResolver_Glue](Classes/classllarp_1_1layers_1_1platform_1_1DNS__NameResolver__Glue.md)** <br>glues together [NameResolver]() and [dns::Resolver_Base](Classes/classllarp_1_1dns_1_1Resolver__Base.md) |
| class | **[llarp::layers::platform::PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md)**  |
| struct | **[llarp::layers::platform::write_to_flow_layer_handler](Classes/structllarp_1_1layers_1_1platform_1_1write__to__flow__layer__handler.md)**  |




## Source code

```cpp
#pragma once

#include "llarp/dns/message.hpp"
#include "llarp/dns/question.hpp"
#include "llarp/dns/rr.hpp"
#include "llarp/layers/flow/flow_info.hpp"
#include "llarp/layers/flow/flow_layer.hpp"

#include "addr_mapper.hpp"
#include "ethertype.hpp"
#include "platform_addr.hpp"
#include "name_resolver.hpp"
#include "traffic.hpp"

#include <llarp/net/ip_packet.hpp>
#include <llarp/util/thread/queue.hpp>

#include <llarp/dns/server.hpp>
#include <llarp/router/abstractrouter.hpp>

#include <llarp/router/route_poker.hpp>
#include <llarp/endpoint_base.hpp>

#include <llarp/layers/layers.hpp>
#include <llarp/vpn/platform.hpp>

#include <memory>
#include <stdexcept>
#include <string_view>

#include <oxenc/variant.h>

namespace llarp::layers::platform
{
  using namespace std::literals;

  class NameLookupJob_Deprecated
  {
    const std::string name;
    const std::shared_ptr<EndpointBase> _endpoint;

   public:
    NameLookupJob_Deprecated(std::shared_ptr<EndpointBase> endpoint_, std::string name_)
        : name{std::move(name_)}, _endpoint{std::move(endpoint_)}
    {}

    void
    resolve_async(std::function<void(std::optional<flow::FlowAddr>)> handle_result) const
    {
      _endpoint->LookupNameAsync(name, [handle_result = std::move(handle_result)](auto maybe_addr) {
        if (not maybe_addr)
        {
          handle_result(std::nullopt);
          return;
        }

        flow::FlowAddr found{var::visit([](auto&& x) { return "{}"_format(x); }, *maybe_addr)};
        handle_result(found);
      });
    }
  };

  struct DNSReplier
  {
    bool wants_localhost_loki;
    bool wants_random_snode;
    dns::Message reply;

    explicit DNSReplier(const dns::Message& msg) : reply{msg}
    {
      const auto& q = msg.questions[0];
      wants_localhost_loki = q.IsLocalhost();
      wants_random_snode = q.IsName("random.snode");
    }
  };

  class DNS_NameResolver_Glue : public dns::Resolver_Base,
                                public std::enable_shared_from_this<DNS_NameResolver_Glue>
  {
    NameResolver& _name_resolver;
    flow::FlowLayer& _flow_layer;
    static constexpr std::array supported_gtld = {".loki"sv, ".snode"sv};

    static bool
    gtld_is_applicable(const dns::Question& question)
    {
      for (auto gtld : supported_gtld)
        if (question.HasTLD(gtld))
          return true;
      return false;
    }

   protected:
    int
    Rank() const override
    {
      return 0;
    }

   public:
    DNS_NameResolver_Glue(NameResolver& name_resolver, flow::FlowLayer& flow_layer)
        : _name_resolver{name_resolver}, _flow_layer{flow_layer}
    {}

    std::string_view
    ResolverName() const override
    {
      return "lokinet";
    }

    bool
    MaybeHookDNS(
        std::shared_ptr<dns::PacketSource_Base> source,
        const dns::Message& query,
        const SockAddr& to,
        const SockAddr& from) override
    {
      // size check
      if (query.questions.size() != 1)
        return false;
      const auto& question = query.questions[0];

      // filter out irrelevent queries
      if (not gtld_is_applicable(question))
        return false;

      DNSReplier dns_reply{query};

      if (dns_reply.wants_localhost_loki)
      {
        return true;
      }

      return false;
    }
  };

  class PlatformDriver_pimpl
  {
   public:
    AbstractRouter& router;
    std::shared_ptr<dns::Resolver_Base> dns_glue;

    bool
    put_exit_up()
    {
      bool has_exit = router.HasClientExit();
      if (const auto& poker = router.routePoker(); poker and has_exit)
        poker->Up();

      return has_exit;
    }

    void
    put_exit_down()
    {
      if (const auto& poker = router.routePoker())
        poker->Down();
    }

    PlatformDriver_pimpl(AbstractRouter& router_) : router{router_}
    {}

    template <typename args_t>
    constexpr auto
    make_waker(args_t&& args)
    {
      return router.loop()->make_waker(args);
    }
  };

  struct write_to_flow_layer_handler
  {
    std::shared_ptr<EndpointBase> _endpoint;
    std::vector<std::vector<byte_t>> datums;
    service::ProtocolType proto;

    void
    operator()(std::optional<service::ConvoTag> maybe_convotag)
    {
      if (maybe_convotag)
      {
        for (auto& data : datums)
          _endpoint->SendToOrQueue(*maybe_convotag, llarp_buffer_t{data}, proto);
      }
      // TODO: handle failure
    }
  };

  namespace
  {
    EndpointBase::AddressVariant_t
    to_addr_variant(flow::FlowAddr addr)
    {
      switch (addr.kind())
      {
        case flow::FlowAddr::Kind::snode:
          return RouterID{std::move(addr.as_array())};
        case flow::FlowAddr::Kind::snapp:
          return service::Address{std::move(addr.as_array())};
        default:
          throw std::invalid_argument{"bad FlowAddr kind"};
      }
    }
  }  // namespace
}  // namespace llarp::layers::platform
```


-------------------------------

Updated on 2023-02-23 at 21:24:54 +0000
