---
title: llarp/layers/flow/deprecated_flow_layer_pimpl.hpp

---

# llarp/layers/flow/deprecated_flow_layer_pimpl.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::flow::Deprecated_SNodeEndpoint](Classes/classllarp_1_1layers_1_1flow_1_1Deprecated__SNodeEndpoint.md)** <br>shim for endpointbase on snode.  |
| struct | **[llarp::layers::flow::DeprecatedPimpl](Classes/structllarp_1_1layers_1_1flow_1_1DeprecatedPimpl.md)** <br>this holds all deprecated types, moved from the old deprecated kitchen sink.  |




## Source code

```cpp
#pragma once
#include "llarp/router_id.hpp"
#ifndef USE_DEPRECATED_LAYERS
#error "not implemented"
#endif
#include "flow_layer.hpp"
#include <llarp/router_contact.hpp>
#include <llarp/service/endpoint.hpp>
#include <llarp/util/logging.hpp>
#include <llarp/router/abstractrouter.hpp>
#include <llarp/dht/context.hpp>

#include <llarp/endpoint_base.hpp>
#include <llarp/exit/session.hpp>
#include <llarp/quic/tunnel.hpp>
#include <llarp/router/i_rc_lookup_handler.hpp>

#include <memory>
#include <stdexcept>

namespace llarp::layers::flow
{
  auto logcat = llarp::log::Cat("layer:flow:deprecated");

  class Deprecated_SNodeEndpoint : public llarp::EndpointBase
  {
    AbstractRouter& _router;

    quic::TunnelManager _quic;

    std::unordered_map<RouterID, std::shared_ptr<exit::SNodeSession>> _sessions;

   public:
    explicit Deprecated_SNodeEndpoint(AbstractRouter& router) : _router{router}, _quic{*this}
    {}

    bool
    LookupRC(RouterID remote, RouterLookupHandler handler) override
    {
      return _router.dht()->impl->LookupRouter(remote, handler);
    }

    std::string_view
    endpoint_name() const override
    {
      return "snode";
    }

    std::optional<EndpointBase::SendStat>
    GetStatFor(AddressVariant_t) const override
    {
      return std::nullopt;
    }

    void
    SRVRecordsChanged() override
    {}

    std::unordered_set<AddressVariant_t>
    AllRemoteEndpoints() const override
    {
      std::unordered_set<AddressVariant_t> addrs{};
      for (const auto& item : _sessions)
        addrs.emplace(item.first);
      return addrs;
    }

    AddressVariant_t
    LocalAddress() const override
    {
      return RouterID{_router.pubkey()};
    }

    quic::TunnelManager*
    GetQUICTunnel() override
    {
      return &_quic;
    }

    std::optional<service::ConvoTag>
    GetBestConvoTagFor(AddressVariant_t) const override
    {
      return std::nullopt;
    }

    bool
    Configure(const NetworkConfig& netconf, const DnsConfig& dnsconf) override
    {
      return true;
    }

    std::optional<AddressVariant_t>
    GetEndpointWithConvoTag(service::ConvoTag tag) const override
    {
      PathID_t id{tag.as_array()};
      for (const auto& item : _sessions)
      {
        if (item.second->GetByEndpointWithID(item.first, id))
          return item.first;
      }
      return std::nullopt;
    }

    const EventLoop_ptr&
    Loop() override
    {
      return _router.loop();
    }

    void
    MarkAddressOutbound(AddressVariant_t) override
    {}

    void
    LookupServiceAsync(
        std::string name,
        std::string service,
        std::function<void(std::vector<dns::SRVData>)> result) override
    {}

    bool
    SendToOrQueue(service::ConvoTag tag, std::vector<byte_t> data, service::ProtocolType t) override
    {
      auto maybe_endpoint = GetEndpointWithConvoTag(tag);
      if (not maybe_endpoint)
        return false;
      auto itr = _sessions.find(std::get<RouterID>(*maybe_endpoint));
      if (itr == _sessions.end())
        return false;
      itr->second->SendPacketToRemote(data, t);
      return true;
    }

    bool
    EnsurePathTo(
        AddressVariant_t remote,
        std::function<void(std::optional<service::ConvoTag>)> hook,
        llarp_time_t timeout) override
    {
      if (std::holds_alternative<service::Address>(remote))
        return false;
      const auto& addr = std::get<RouterID>(remote);
      // check for valid service node
      if (not _router.rcLookupHandler().PathIsAllowed(addr))
        return false;

      autovivify_session_to(addr)->AddReadyHook([hook = std::move(hook)](auto ptr) {
        if (ptr)
        {
          if (auto path = ptr->GetPathByRouter(ptr->Endpoint()))
          {
            hook(service::ConvoTag{path->EndpointHop().txID});
            return;
          }
        }
        hook(std::nullopt);
      });
      return true;
    }

    const std::shared_ptr<exit::SNodeSession>&
    autovivify_session_to(RouterID remote)
    {
      auto [itr, inserted] = _sessions.try_emplace(remote);
      if (inserted)
      {
        auto write_packet = [](auto) {};
        constexpr auto num_paths = 2;
        constexpr auto hop_len = 1;
        itr->second = std::make_shared<exit::SNodeSession>(
            remote, write_packet, &_router, num_paths, 1, this);
      }
      return itr->second;
    }
  };

  struct DeprecatedPimpl
  {
    AbstractRouter& _router;
    std::shared_ptr<llarp::EndpointBase> endpoint;

    explicit DeprecatedPimpl(AbstractRouter& router) : _router{router}
    {
      if (router.IsServiceNode())
      {
        endpoint = std::make_shared<Deprecated_SNodeEndpoint>(_router);
      }
      else
      {
        endpoint = std::make_shared<llarp::service::Endpoint>(_router);
      }
      std::weak_ptr weak{endpoint};

      // set up end of event loop pump
      _router.loop()->add_ticker([weak]() {
        if (auto ptr = weak.lock())
        {
          log::trace(logcat, "pump '{}'"_format(ptr->endpoint_name()));
          ptr->event_loop_pump();
        }
      });

      // call periodic tick function.
      // this was done before in service::Context and exit::Context (i think)
      _router.loop()->call_every(100ms, weak, [this]() {
        if (not endpoint)
          return;
        log::trace(logcat, "tick '{}'"_format(endpoint->endpoint_name()));
        endpoint->periodic_tick();
      });

      log::info(logcat, "configuring...");
      auto config = _router.GetConfig();
      if (not endpoint->Configure(config->network, config->dns))
        throw std::runtime_error{"ep->Configure(netconf, dnsconf) returned false"};

      log::info(logcat, "set up '{}'"_format(endpoint->endpoint_name()));
    }

    void
    start()
    {
      endpoint->start_endpoint();
    }

    void
    stop()
    {
      endpoint->stop_endpoint();
      endpoint = nullptr;
    }
  };
}  // namespace llarp::layers::flow
```


-------------------------------

Updated on 2023-02-23 at 21:24:54 +0000
