---
title: llarp/layers/flow/flow_layer.hpp

---

# llarp/layers/flow/flow_layer.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::flow::FlowLayer](Classes/classllarp_1_1layers_1_1flow_1_1FlowLayer.md)** <br>flow layer's context that holds all the things in the flow layer  |




## Source code

```cpp
#pragma once

#include "flow_addr.hpp"
#include "flow_identity.hpp"
#include "flow_tag.hpp"
#include "flow_stats.hpp"
#include "flow_traffic.hpp"

#include "name_cache.hpp"
#include "name_resolver.hpp"

#include <llarp/config/config.hpp>
#include <memory>
#include <vector>

namespace llarp
{
  struct AbstractRouter;
}

namespace llarp::layers::flow
{

  class FlowLayer
  {
    const NetworkConfig _conf;
    std::vector<std::unique_ptr<FlowIdentity>> _local_flows;

    FlowIdentityPrivateKeys _privkeys;

    NameCache _name_cache;

    void
    maybe_store_or_load_privkeys();

    FlowTag
    unique_flow_tag() const;

    bool
    has_flow_tag(const FlowTag&) const;

   public:
    FlowLayer(AbstractRouter&, NetworkConfig);
    ~FlowLayer();

    FlowLayer(const FlowLayer&) = delete;
    FlowLayer(FlowLayer&&) = delete;

    FlowStats
    current_stats() const;

    inline FlowAddr
    local_addr() const
    {
      return current_stats().local_addr;
    }

    FlowIdentity&
    best_flow_to(const FlowAddr& addr);

    // FlowIdentity *
    // get_source_for(const FlowTag& tag);

    void
    offer_flow_traffic(FlowTraffic&& traff);

    void
    start();

    void
    stop();

    NameResolver name_resolver;

    AbstractRouter& router;
  };
}  // namespace llarp::layers::flow
```


-------------------------------

Updated on 2023-02-23 at 21:24:54 +0000
